unit SjrdEdits;

interface

uses
  Windows, Messages, SysUtils, Classes, SjrdUsual, StdCtrls;

type
  TSjrdNumberEdit = class(TCustomEdit)
  private
    { Dclarations prives }
    FPrefixe, FSufixe : string;
    FNegatifOK, FDecimalOK : boolean;
    FSeparator : Char;
    FHasMinValue, FHasMaxValue : boolean;
    FMinValue, FMaxValue : Double;
    function GetValue : Double;
    function GetInt : integer;
    procedure SetValue(New : Double);
    procedure SetInt(New : integer);
    procedure SetPrefixe(New : string);
    procedure SetSufixe(New : string);
    procedure SetNegatifOK(New : boolean);
    procedure SetDecimalOK(New : boolean);
    procedure SetSeparator(New : Char);
    procedure SetHasMinValue(New : boolean);
    procedure SetHasMaxValue(New : boolean);
    procedure SetMinValue(New : Double);
    procedure SetMaxValue(New : Double);

    function SelEnd : integer;
  protected
    { Dclarations protges }
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
  public
    { Dclarations publiques }
    constructor Create(AOwner : TComponent); override;
    property Int : integer read GetInt write SetInt;
  published
    { Dclarations publies }
    property AutoSelect default False;
    property AutoSize;
    property BorderStyle;
    property HideSelection;
    property ReadOnly;
    property TabOrder;
    property Action;
    property Align;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DockOrientation;
    property Enabled;
    property FloatingDockSiteClass;
    property ParentColor;
    property ShowHint;
    property Visible;

    property Value : Double read GetValue write SetValue;
    property Prefixe : string read FPrefixe write SetPrefixe;
    property Sufixe  : string read FSufixe  write SetSufixe;
    property NegatifOK : boolean read FNegatifOK write SetNegatifOK default False;
    property DecimalOK : boolean read FDecimalOK write SetDecimalOK default False;
    property Separator : Char read FSeparator write SetSeparator default ',';
    property HasMinValue : boolean read FHasMinValue write SetHasMinValue default False;
    property HasMaxValue : boolean read FHasMaxValue write SetHasMaxValue default False;
    property MinValue : Double read FMinValue write SetMinValue;
    property MaxValue : Double read FMaxValue write SetMaxValue;

    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnEnter;
    property OnExit;
    property OnDockDrop;
    property OnDockOver;
    property OnGetSiteInfo;
    property OnUnDock;
    property OnCanResize;
    property OnConstrainedResize;
    property OnResize;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnStartDock;
    property OnEndDock;
    property OnStartDrag;
    property OnEndDrag;
  end;

implementation

//////////////////////////////
/// Classe TSjrdNumberEdit ///
//////////////////////////////

constructor TSjrdNumberEdit.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  AutoSelect := False;
  FPrefixe := '';
  FSufixe  := '';
  FNegatifOK := False;
  FDecimalOK := False;
  FSeparator := ',';
  FHasMinValue := False;
  FHasMaxValue := False;
  FMinValue := 0.0;
  FMaxValue := 0.0;
  Int := 0;
end;

function TSjrdNumberEdit.GetValue : Double;
var TextLen : integer;
    Text : PChar;
    Str : string;
begin
  TextLen := SendMessage(Handle, WM_GetTextLength, 0, 0);
  SetLength(Str, TextLen);
  Text := PChar(Str);
  SendMessage(Handle, WM_GetText, TextLen, LParam(Text));
  Str := Text;
  if Length(Str) <= ( Length(FPrefixe) + Length(FSufixe) ) then Result := 0.0 else
  if Copy(Str, 1, Length(FPrefixe)) <> FPrefixe then Result := 0.0 else
  if Copy(Str, Length(Str)-Length(FSufixe)+1, Length(FSufixe)) <> FSufixe then Result := 0.0 else
  begin
    Str := Copy(Str, Length(FPrefixe)+1, Length(Str)-Length(FPrefixe)-Length(FSufixe));
    if Pos(FSeparator, Str) > 0 then Str[Pos(FSeparator, Str)] := DecimalSeparator;
    try
      Result := StrToFloat(Str);
    except
      on E : EConvertError do Result := 0.0;
    end;
    if FHasMinValue and (Result < FMinValue) then Result := FMinValue;
    if FHasMaxValue and (Result > FMaxValue) then Result := FMaxValue;
  end;
end;

function TSjrdNumberEdit.GetInt : integer;
begin
  Result := Round(Value);
end;

procedure TSjrdNumberEdit.SetValue(New : Double);
var Str : string;
    Text : PChar;
begin
  if csReading in ComponentState then exit;
  if not NegatifOK then New := Abs(New);
  if not DecimalOK then New := Round(New);
  if FHasMinValue and (New < FMinValue) then New := FMinValue;
  if FHasMaxValue and (New > FMaxValue) then New := FMaxValue;
  Str := FloatToStr(New);
  if Pos(DecimalSeparator, Str) > 0 then Str[Pos(DecimalSeparator, Str)] := FSeparator;
  Text := PChar(Prefixe + Str + Sufixe);
  SendMessage(Handle, WM_SetText, 0, LParam(Text));
end;

procedure TSjrdNumberEdit.SetInt(New : integer);
begin
  Value := New;
end;

procedure TSjrdNumberEdit.SetPrefixe(New : string);
var Anc : Double;
begin
  Anc := Value;
  FPrefixe := New;
  if not (csReading in ComponentState) then Value := Anc;
end;

procedure TSjrdNumberEdit.SetSufixe(New : string);
var Anc : Double;
begin
  Anc := Value;
  FSufixe := New;
  if not (csReading in ComponentState) then Value := Anc;
end;

procedure TSjrdNumberEdit.SetNegatifOK(New : boolean);
var Anc : Double;
begin
  Anc := Value;
  FNegatifOK := New;
  if not (csReading in ComponentState) then
    if New then Value := Anc else Value := Abs(Anc);
end;

procedure TSjrdNumberEdit.SetDecimalOK(New : boolean);
var Anc : Double;
begin
  Anc := Value;
  FDecimalOK := New;
  if not (csReading in ComponentState) then
    if New then Value := Anc else Value := Round(Anc);
end;

procedure TSjrdNumberEdit.SetSeparator(New : Char);
var Anc : Double;
begin
  Anc := Value;
  FSeparator := New;
  if not (csReading in ComponentState) then Value := Anc;
end;

procedure TSjrdNumberEdit.SetHasMinValue(New : boolean);
begin
  if New = FHasMinValue then exit;
  FHasMinValue := New;
  MaxValue := MaxValue;
  Value := Value;
end;

procedure TSjrdNumberEdit.SetHasMaxValue(New : boolean);
begin
  if New = FHasMaxValue then exit;
  FHasMaxValue := New;
  MinValue := MinValue;
  Value := Value;
end;

procedure TSjrdNumberEdit.SetMinValue(New : Double);
begin
  if FHasMaxValue and (New > FMaxValue) then FMinValue := FMaxValue else FMinValue := New;
  Value := Value;
end;

procedure TSjrdNumberEdit.SetMaxValue(New : Double);
begin
  if FHasMinValue and (New < FMinValue) then FMaxValue := FMinValue else FMaxValue := New;
  Value := Value;
end;

function TSjrdNumberEdit.SelEnd : integer;
begin
  Result := SelStart + SelLength;
end;

procedure TSjrdNumberEdit.KeyDown(var Key : Word; Shift : TShiftState);
const IgnoredKeys =
  [VK_Tab, VK_Return, VK_Shift, VK_Control, VK_Menu, VK_Pause, VK_Capital,
  VK_Space, VK_Prior, VK_Next, VK_End, VK_Home, VK_Left, VK_Up, VK_Right,
  VK_Down];
var Debut, Fin : integer;
begin
  if ReadOnly then exit;
  if Key in IgnoredKeys then exit;
  Debut := Length(FPrefixe);
  Fin := Length(Text)-Length(FSufixe);
  if (SelStart < Debut) or (SelEnd > Fin) then Key := 0 else
  if (SelStart = Fin) and (SelEnd = Fin) and (Key = VK_Delete) then Key := 0;
  if Key = 0 then ExecuteSound('TuutTut');
end;

procedure TSjrdNumberEdit.KeyPress(var Key: Char);
var Debut, Fin : integer;
begin
  if ReadOnly then exit;
  Debut := Length(FPrefixe);
  Fin := Length(Text)-Length(FSufixe);
  if (SelStart < Debut) or (SelEnd > Fin) then Key := #0 else
  if (SelStart = Debut) and (SelEnd = Debut) and (Key = Char(VK_Back)) then Key := #0 else
  if (Key = '-') and ((not FNegatifOK) or (SelStart > Debut)) then Key := #0 else
  if ((Key = '.') or (Key = ',')) and ((not DecimalOK) or (Pos(FSeparator, Copy(Text, Length(FPrefixe)+1, Length(Text)-Length(FPrefixe)-Length(FSufixe))) > 0)) then Key := #0 else
  if ((Key < '0') or (Key > '9')) and
     (Key <> '.') and (Key <> ',') and (Key <> '-') and
     (Key <> Char(VK_Back)) then Key := #0 else
  if (Key = '.') or (Key = ',') then Key := FSeparator;
end;

end.
