unit SjrdImages;

interface

uses
  Windows, Forms, Classes, Controls, ExtCtrls;

type
  /// Types des vnements
  TPutEvent = procedure(Sender: TObject; X, Y : integer) of object;

  /// Autres types

  /// Composants
  TSjrdMoveImage = class(TImage)
  private
    ImageBis : TImage;
    FPutControl : TControl;
    FOnPut : TPutEvent;
    procedure CalculePos(X, Y : integer);
  protected
    procedure MouseDown(Button : TMouseButton; Shift : TShiftState; X, Y : integer); override;
    procedure MouseMove(Shift : TShiftState; X, Y : integer); override;
    procedure MouseUp  (Button : TMouseButton; Shift : TShiftState; X, Y : integer); override;
  public
    constructor Create(AOwner : TComponent); override;
  published
    property PutControl : TControl read FPutControl write FPutControl default nil;
    property OnPut : TPutEvent read FOnPut write FOnPut default nil;
  end;

implementation

/////////////////////////////
/// Classe TSjrdMoveImage ///
/////////////////////////////

constructor TSjrdMoveImage.Create(AOwner : TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csCaptureMouse];
  ImageBis := nil;
  FPutControl := nil;
  FOnPut := nil;
end;

procedure TSjrdMoveImage.CalculePos(X, Y : integer);
var L, T, ParentWidth, ParentHeight : integer;
begin
  if Parent is TForm then
  begin
    ParentWidth  := (Parent as TForm).ClientWidth ;
    ParentHeight := (Parent as TForm).ClientHeight;
  end else
  begin
    ParentWidth  := Parent.Width ;
    ParentHeight := Parent.Height;
  end;
  L := Left + X - Width  div 2;
  T := Top  + Y - Height div 2;
  if L > (ParentWidth -Width ) then L := ParentWidth -Width ;
  if L < 0 then L := 0;
  if T > (ParentHeight-Height) then T := ParentHeight-Height;
  if T < 0 then T := 0;
  ImageBis.Left := L;
  ImageBis.Top  := T;
end;

procedure TSjrdMoveImage.MouseDown(Button : TMouseButton; Shift : TShiftState; X, Y : integer);
begin
  inherited;
  if not Enabled then exit;
  if (Button <> mbLeft) or Assigned(ImageBis) then exit;
  ImageBis := TImage.Create(Self);
  ImageBis.Assign(Self);
  CalculePos(X, Y);
end;

procedure TSjrdMoveImage.MouseMove(Shift : TShiftState; X, Y : integer);
begin
  inherited;
  if Assigned(ImageBis) then CalculePos(X, Y);
end;

procedure TSjrdMoveImage.MouseUp(Button : TMouseButton; Shift : TShiftState; X, Y : integer);
var PutPoint : TPoint;
begin
  inherited;
  if (Button <> mbLeft) or (not Assigned(ImageBis)) then exit;
  ImageBis.Free;
  ImageBis := nil;
  if not Assigned(FOnPut) then exit;
  if Assigned(FPutControl) then
    PutPoint := FPutControl.ScreenToClient(ClientToScreen(Point(X, Y)))
  else
    PutPoint := ClientToParent(Point(X, Y));
  FOnPut(Self, PutPoint.X, PutPoint.Y);
end;

end.
