unit SjrdLabels;

interface

{$I SJRDComps.inc}

uses
{$IFDEF SJRD_WINDOWS}
  Controls, Graphics, StdCtrls, Menus, ClipBrd,
{$ENDIF}
{$IFDEF SJRD_LINUX}
  QControls, QGraphics, QStdCtrls, QMenus, QClipBrd,
{$ENDIF}
  Classes, SjrdUsual;

type
  TRunURLEvent = procedure(Sender : TObject; URL : string) of object;

  TSjrdURLLabel = class(TLabel)
  private
    FURL : string;
    Menu : TPopupMenu;
    FOnRunURL : TRunURLEvent;
    function RealURL : string;
    procedure MenuCopierClick(Sender : TObject);
    procedure MenuLancerClick(Sender : TObject);
    function IsFontStored : boolean;
    function IsPopupMenuStored : boolean;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
  published
    property Cursor default crHandPoint;
    property Font stored IsFontStored;
    property PopupMenu stored IsPopupMenuStored;
    property URL : string read FURL write FURL;

    property OnRunURL : TRunURLEvent read FOnRunURL write FOnRunURL;
      // CLX uniquement : doit lancer l'URL passe en paramtre
  end platform;

implementation

////////////////////////////
/// Classe TSjrdURLLabel ///
////////////////////////////

constructor TSjrdURLLabel.Create(AOwner : TComponent);
var TempMenu : TMenuItem;
begin
   inherited Create(AOwner);
   Cursor := crHandPoint;
   with Font do
   begin
     Color := clBlue;
     Style := [fsUnderline];
   end;
   Menu := TPopupMenu.Create(Self);
   Menu.Name := 'PopupMenu';
   TempMenu := TMenuItem.Create(Menu);
   with TempMenu do
   begin
     Name := 'MenuCopy';
     Caption := 'Copier l''URL';
     Hint := 'Copier l''URL dans le presse-papier';
     OnClick := MenuCopierClick;
   end;
   Menu.Items.Add(TempMenu);
   TempMenu := TMenuItem.Create(Menu);
   with TempMenu do
   begin
     Name := 'MenuLancer';
     Caption := 'Lancer';
     Hint := 'Lancer l''URL';
     OnClick := MenuLancerClick;
   end;
   Menu.Items.Add(TempMenu);
   PopupMenu := Menu;
end;

function TSjrdURLLabel.RealURL : string;
begin
  if URL = '' then Result := Caption else Result := URL;
end;

destructor TSjrdURLLabel.Destroy;
begin
   Menu.Free;
   inherited;
end;

procedure TSjrdURLLabel.MenuCopierClick;
begin
  Clipboard.AsText := RealURL;
end;

procedure TSjrdURLLabel.MenuLancerClick(Sender : TObject);
begin
  Click;
end;

function TSjrdURLLabel.IsFontStored : boolean;
begin
  Result := True;
end;

function TSjrdURLLabel.IsPopupMenuStored : boolean;
begin
  Result := PopupMenu <> Menu;
end;

procedure TSjrdURLLabel.Click;
begin
  inherited Click;
  {$IFDEF SJRD_WINDOWS}
    RunURL(RealURL);
  {$ENDIF}
  {$IFDEF SJRD_LINUX}
    if Assigned(FOnRunURL) then FOnRunURL(Self, RealURL);
  {$ENDIF}
end;

end.
