unit SjrdNonVisual;

interface

uses
  Classes, Graphics, SjrdPassWord, SjrdAbout;

type
  TSjrdPassWordDialog = class(TComponent)
  private
    FPassWord : string;
    FShowErrorMes : boolean;
  public
    constructor Create(AOwner : TComponent); override;
    function Execute : boolean; overload;
    function Execute(PassWord : string; ShowErrorMes : boolean = True) : boolean; overload;
    function Execute(ShowErrorMes : boolean) : boolean; overload;
  published
    property PassWord : string read FPassWord write FPassWord;
    property ShowErrorMes : boolean read FShowErrorMes write FShowErrorMes default True;
  end;

  TSjrdAboutDialog = class(TComponent)
  private
    FTitle : string;
    FProgramIcon : TIcon;
    FProgramName : string;
    FVersion : string;
    FProgramVersion : string;
    FAuthor : string;
    FAuthorName : string;
    FAuthorEMail : string;
    FWebSite : string;
    procedure SetProgramIcon(New : TIcon);
    function IsTitleStored : boolean;
    function IsVersionStored : boolean;
    function IsProgramVersionStored : boolean;
    function IsAuthorStored : boolean;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Execute;
  published
    property Title : string read FTitle write FTitle stored IsTitleStored;
    property ProgramIcon : TIcon read FProgramIcon write SetProgramIcon;
    property ProgramName : string read FProgramName write FProgramName;
    property Version : string read FVersion write FVersion stored IsVersionStored;
    property ProgramVersion : string read FProgramVersion write FProgramVersion stored IsProgramVersionStored;
    property Author : string read FAuthor write FAuthor stored IsAuthorStored;
    property AuthorName : string read FAuthorName write FAuthorName;
    property AuthorEMail : string read FAuthorEMail write FAuthorEMail;
    property WebSite : string read FWebSite write FWebSite;
  end platform;

implementation

//////////////////////////////////
/// Classe TSjrdPassWordDialog ///
//////////////////////////////////

constructor TSjrdPassWordDialog.Create(AOwner : TComponent);
begin
  inherited;
  FPassWord := '';
  FShowErrorMes := True;
end;

function TSjrdPassWordDialog.Execute : boolean;
begin
  Result := Execute(PassWord, ShowErrorMes);
end;

function TSjrdPassWordDialog.Execute(PassWord : string; ShowErrorMes : boolean = True) : boolean;
begin
  Result := TSjrdPassWordForm.QueryPassWord(PassWord, ShowErrorMes);
end;

function TSjrdPassWordDialog.Execute(ShowErrorMes : boolean) : boolean;
begin
  Result := Execute(PassWord, ShowErrorMes);
end;

///////////////////////////////
/// Classe TSjrdAboutDialog ///
///////////////////////////////

constructor TSjrdAboutDialog.Create(AOwner : TComponent);
begin
  inherited;
  FTitle := 'A propos';
  FProgramIcon := TIcon.Create;
  FProgramName := '';
  FVersion := 'Version';
  FProgramVersion := '1.0';
  FAuthor := 'Auteur';
  FAuthorName := '';
  FAuthorEMail := '';
  FWebSite := '';
end;

destructor TSjrdAboutDialog.Destroy;
begin
  FProgramIcon.Free;
  inherited;
end;

procedure TSjrdAboutDialog.SetProgramIcon(New : TIcon);
begin
  FProgramIcon.Assign(New);
end;

function TSjrdAboutDialog.IsTitleStored : boolean;
begin
  Result := FTitle <> 'A propos';
end;

function TSjrdAboutDialog.IsVersionStored : boolean;
begin
  Result := FVersion <> 'Version';
end;

function TSjrdAboutDialog.IsProgramVersionStored : boolean;
begin
  Result := FProgramVersion <> '1.0';
end;

function TSjrdAboutDialog.IsAuthorStored : boolean;
begin
  Result := FAuthor <> 'Auteur';
end;

procedure TSjrdAboutDialog.Execute;
begin
  TSjrdAboutForm.ShowAbout(Title, ProgramIcon, ProgramName,
                           Version+' : '+ProgramVersion,
                           Author+' : '+AuthorName,
                           AuthorEMail, WebSite);
end;

end.
